# -*- coding: utf-8 -*- 
# @Time    : 03/22/2024 
# @Author  : nzooherd 
# @File    : video_dao.py 
# @Software: Visual Studio Code


from typing import List
from sqlalchemy import func, select

from dao import BaseDao
from dao.model import Purchase, Video, VideoTags


class VideoDao(BaseDao):

    model = Video

    @classmethod
    async def get_video_by_md5(cls, md5: str):
        async with cls.session() as session:
            query = select(cls.model).filter_by(md5=md5)
            result = await session.execute(query)
            video = result.scalars().first()
            if not video:
                return None
            session.expunge(video)
            return video
    
    @classmethod
    async def query_video_by_bangou(cls, bangou: str) -> List[Video]:
        async with cls.session() as session:
            query = select(cls.model).filter_by(bangou=bangou)
            result = await session.execute(query)
            video = result.scalars().all()
            if not video:
                return []
            session.expunge_all()
            return video

    @classmethod
    async def query_video_by_filename(cls, filename: str) -> List[Video]:
        async with cls.session() as session:
            query = select(cls.model).filter_by(filename=filename)
            result = await session.execute(query)
            video = result.scalars().all()
            if not video:
                return []
            session.expunge_all()
            return video



    @classmethod
    async def update_attribute(cls, md5: str, **kwargs):
        async with cls.session() as session:
            query = select(cls.model).filter_by(md5=md5)
            result = await session.execute(query)
            video = result.scalars().first()
            for key, value in kwargs.items():
                setattr(video, key, value)
    
    @classmethod
    async def activate_count(cls) -> int:
        async with cls.session() as session:
            query = select(func.count("*").label("count")).select_from(cls.model).filter_by(status=1)
            result = await session.execute(query)
            return result.scalars().first()
    

    @classmethod
    async def select_by_offset(cls, offset: int) -> Video:
        async with cls.session() as session:
            query = select(cls.model).order_by(cls.model.id.desc()).\
                filter_by(status=1).\
                offset(offset).\
                limit(1)
            result = await session.execute(query)
            video = result.scalars().first()
            session.expunge(video)
            return video
    
    @classmethod
    async def select_has_bought(cls, uid: int) -> List:
        async with cls.session() as session:
            query = select(cls.model.id).join(Purchase, cls.model.id == Purchase.video_id).\
                filter(Purchase.user_id == uid)
            result = await session.execute(query)
            video_ids = result.scalars().all()
            session.expunge_all()
            return video_ids
    
    @classmethod
    async def select_by_tags(cls, tags: List[str]) -> List[Video]:
        async with cls.session() as session:
            query = select(cls.model.id).join(VideoTags, VideoTags.video_id == cls.model.id).\
                filter(VideoTags.tag.in_(tags)).\
                filter(cls.model.status == 1)
            result = await session.execute(query)
            videos = result.scalars().all()
            session.expunge_all()
            return videos
    
    @classmethod
    async def select_video_scorll(cls, video_id: int, scroll_dir: int) -> Video:
        async with cls.session() as session:
            if (scroll_dir == 1):
                query = select(cls.model).filter(cls.model.id < video_id).order_by(cls.model.id.desc()).limit(1)
            else:
                query = select(cls.model).filter(cls.model.id > video_id).order_by(cls.model.id.asc()).limit(1)
            result = await session.execute(query)
            video = result.scalars().first()
            session.expunge(video)
            return video